unit GameViewMain;

interface

uses Classes,
  CastleVectors, CastleComponentSerialize,
  CastleUIControls, CastleControls, CastleKeysMouse, CastleScene, CastleViewport,  X3DNodes, X3DFields;

type
  { Main view, where most of the application logic takes place. }
  TViewMain = class(TCastleView)
  published
    { Components designed using CGE editor.
      These fields will be automatically initialized at Start. }
    Viewport: TCastleViewport;
    Scene1: TCastleScene;
    Label1: TCastleLabel;
  private
    EffectColorField: TSFVec3f;
    procedure LightupColor(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    procedure Start; override;
    procedure Update(const SecondsPassed: Single; var HandleInput: Boolean); override;
    function Press(const Event: TInputPressRelease): Boolean; override;
  end;

var
  ViewMain: TViewMain;

implementation

uses SysUtils, CastleRenderOptions;

{ TViewMain ----------------------------------------------------------------- }

constructor TViewMain.Create(AOwner: TComponent);
begin
  inherited;
  DesignUrl := 'castle-data:/gameviewmain.castle-user-interface';
end;

procedure TViewMain.Start;

  procedure CreateColorEffect(const Scene: TCastleScene);
  var
    Effect: TEffectNode;
    EffectPart: TEffectPartNode;
  begin
    Effect := TEffectNode.Create;
    Effect.Language := slGLSL;

    EffectColorField := TSFVec3f.Create(Effect, true, 'color', Vector3(1, 1, 1));
    Effect.AddCustomField(EffectColorField);

    EffectPart := TEffectPartNode.Create;
    EffectPart.ShaderType := stFragment;
    EffectPart.SetUrl(['castle-data:/shaders/color_effect.fs']);
    Effect.SetParts([EffectPart]);

    Scene.RootNode.AddChildren([Effect]);

    Scene.PlayAnimation('jackfacesw', true);
  end;

begin
  inherited;

  CreateColorEffect(Scene1);

end;

procedure TViewMain.Update(const SecondsPassed: Single; var HandleInput: Boolean);
begin
  inherited;

  if Viewport.TransformUnderMouse <> nil then
  begin
    Label1.Caption := Viewport.TransformUnderMouse.Name;
    ViewMain.LightupColor(Scene1);
  end
  else
  begin
    EffectColorField.Send(Vector3(1, 1, 1));

   Label1.Caption := ' ';
  end;
end;

procedure TViewMain.LightupColor(Sender: TObject);
var
  NewColor: TVector3;
begin
  NewColor := Vector3(1.3, 1.3, 1.3);
  EffectColorField.Send(NewColor);

end;

function TViewMain.Press(const Event: TInputPressRelease): Boolean;
begin
  Result := inherited;
  if Result then Exit;
end;

end.
